@@ -18,11 +18,12 @@
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: avrpart.c 1436 2020-09-10 21:37:34Z joerg_wunsch $ */
+/* $Id$ */
 
 #include <stdlib.h>
 #include <string.h>
 
+#include "ac_cfg.h"
 #include "avrdude.h"
 #include "libavrdude.h"
 
@@ -259,6 +260,21 @@ AVRMEM * avr_new_memtype(void)
   return m;
 }
 
+AVRMEM_ALIAS * avr_new_memalias(void)
+{
+  AVRMEM_ALIAS * m;
+
+  m = (AVRMEM_ALIAS *)malloc(sizeof(*m));
+  if (m == NULL) {
+    avrdude_message(MSG_INFO, "avr_new_memalias(): out of memory\n");
+    exit(1);
+  }
+
+  memset(m, 0, sizeof(*m));
+
+  return m;
+}
+
 
 /*
  * Allocate and initialize memory buffers for each of the device's
@@ -325,6 +341,17 @@ AVRMEM * avr_dup_mem(AVRMEM * m)
   return n;
 }
 
+AVRMEM_ALIAS * avr_dup_memalias(AVRMEM_ALIAS * m)
+{
+  AVRMEM_ALIAS * n;
+
+  n = avr_new_memalias();
+
+  *n = *m;
+
+  return n;
+}
+
 void avr_free_mem(AVRMEM * m)
 {
     int i;
@@ -347,7 +374,36 @@ void avr_free_mem(AVRMEM * m)
     free(m);
 }
 
-AVRMEM * avr_locate_mem(AVRPART * p, char * desc)
+void avr_free_memalias(AVRMEM_ALIAS * m)
+{
+  free(m);
+}
+
+AVRMEM_ALIAS * avr_locate_memalias(AVRPART * p, char * desc)
+{
+  AVRMEM_ALIAS * m, * match;
+  LNODEID ln;
+  int matches;
+  int l;
+
+  l = strlen(desc);
+  matches = 0;
+  match = NULL;
+  for (ln=lfirst(p->mem_alias); ln; ln=lnext(ln)) {
+    m = ldata(ln);
+    if (strncmp(desc, m->desc, l) == 0) {
+      match = m;
+      matches++;
+    }
+  }
+
+  if (matches == 1)
+    return match;
+
+  return NULL;
+}
+
+AVRMEM * avr_locate_mem_noalias(AVRPART * p, char * desc)
 {
   AVRMEM * m, * match;
   LNODEID ln;
@@ -372,38 +428,97 @@ AVRMEM * avr_locate_mem(AVRPART * p, char * desc)
 }
 
 
-void avr_mem_display(const char * prefix, FILE * f, AVRMEM * m, int type,
-                     int verbose)
+AVRMEM * avr_locate_mem(AVRPART * p, char * desc)
 {
+  AVRMEM * m, * match;
+  AVRMEM_ALIAS * alias;
+  LNODEID ln;
+  int matches;
+  int l;
+
+  l = strlen(desc);
+  matches = 0;
+  match = NULL;
+  for (ln=lfirst(p->mem); ln; ln=lnext(ln)) {
+    m = ldata(ln);
+    if (strncmp(desc, m->desc, l) == 0) {
+      match = m;
+      matches++;
+    }
+  }
+
+  if (matches == 1)
+    return match;
+
+  /* not yet found: look for matching alias name */
+  alias = avr_locate_memalias(p, desc);
+  if (alias != NULL)
+    return alias->aliased_mem;
+
+  return NULL;
+}
+
+AVRMEM_ALIAS * avr_find_memalias(AVRPART * p, AVRMEM * m_orig)
+{
+  AVRMEM_ALIAS * m;
+  LNODEID ln;
+
+  for (ln=lfirst(p->mem_alias); ln; ln=lnext(ln)) {
+    m = ldata(ln);
+    if (m->aliased_mem == m_orig)
+      return m;
+  }
+
+  return NULL;
+}
+
+
+void avr_mem_display(const char * prefix, FILE * f, AVRMEM * m, AVRPART * p,
+                     int type, int verbose)
+{
+  static unsigned int prev_mem_offset, prev_mem_size;
   int i, j;
   char * optr;
 
   if (m == NULL) {
       fprintf(f,
-              "%s                       Block Poll               Page                       Polled\n"
-              "%sMemory Type Mode Delay Size  Indx Paged  Size   Size #Pages MinW  MaxW   ReadBack\n"
-              "%s----------- ---- ----- ----- ---- ------ ------ ---- ------ ----- ----- ---------\n",
+              "%s                                Block Poll               Page                       Polled\n"
+              "%sMemory Type Alias    Mode Delay Size  Indx Paged  Size   Size #Pages MinW  MaxW   ReadBack\n"
+              "%s----------- -------- ---- ----- ----- ---- ------ ------ ---- ------ ----- ----- ---------\n",
             prefix, prefix, prefix);
   }
   else {
     if (verbose > 2) {
       fprintf(f,
-              "%s                       Block Poll               Page                       Polled\n"
-              "%sMemory Type Mode Delay Size  Indx Paged  Size   Size #Pages MinW  MaxW   ReadBack\n"
-              "%s----------- ---- ----- ----- ---- ------ ------ ---- ------ ----- ----- ---------\n",
+              "%s                                Block Poll               Page                       Polled\n"
+              "%sMemory Type Alias    Mode Delay Size  Indx Paged  Size   Size #Pages MinW  MaxW   ReadBack\n"
+              "%s----------- -------- ---- ----- ----- ---- ------ ------ ---- ------ ----- ----- ---------\n",
               prefix, prefix, prefix);
     }
-    fprintf(f,
-            "%s%-11s %4d %5d %5d %4d %-6s %6d %4d %6d %5d %5d 0x%02x 0x%02x\n",
-            prefix, m->desc, m->mode, m->delay, m->blocksize, m->pollindex,
-            m->paged ? "yes" : "no",
-            m->size,
-            m->page_size,
-            m->num_pages,
-            m->min_write_delay,
-            m->max_write_delay,
-            m->readback[0],
-            m->readback[1]);
+
+    // Only print memory section if the previous section printed isn't identical
+    if(prev_mem_offset != m->offset || prev_mem_size != m->size || (strcmp(p->family_id, "") == 0)) {
+      prev_mem_offset = m->offset;
+      prev_mem_size = m->size;
+      AVRMEM_ALIAS *ap = avr_find_memalias(p, m);
+      /* Show alias if the current and the next memory section has the same offset
+      and size, we're not out of band and a family_id is present */
+      char * mem_desc_alias = ap? ap->desc: "";
+      fprintf(f,
+              "%s%-11s %-8s %4d %5d %5d %4d %-6s %6d %4d %6d %5d %5d 0x%02x 0x%02x\n",
+              prefix,
+              m->desc,
+              mem_desc_alias,
+              m->mode, m->delay, m->blocksize, m->pollindex,
+              m->paged ? "yes" : "no",
+              m->size,
+              m->page_size,
+              m->num_pages,
+              m->min_write_delay,
+              m->max_write_delay,
+              m->readback[0],
+              m->readback[1]);
+    }
     if (verbose > 4) {
       avrdude_message(MSG_TRACE2, "%s  Memory Ops:\n"
                       "%s    Oeration     Inst Bit  Bit Type  Bitno  Value\n"
@@ -460,6 +575,7 @@ AVRPART * avr_new_part(void)
   p->ocdrev = -1;
 
   p->mem = lcreat(NULL, 0);
+  p->mem_alias = lcreat(NULL, 0);
 
   return p;
 }
@@ -468,19 +584,35 @@ AVRPART * avr_new_part(void)
 AVRPART * avr_dup_part(AVRPART * d)
 {
   AVRPART * p;
-  LISTID save;
-  LNODEID ln;
+  LISTID save, save2;
+  LNODEID ln, ln2;
   int i;
 
   p = avr_new_part();
   save = p->mem;
+  save2 = p->mem_alias;
 
   *p = *d;
 
   p->mem = save;
+  p->mem_alias = save2;
 
   for (ln=lfirst(d->mem); ln; ln=lnext(ln)) {
-    ladd(p->mem, avr_dup_mem(ldata(ln)));
+    AVRMEM *m = ldata(ln);
+    AVRMEM *m2 = avr_dup_mem(m);
+    ladd(p->mem, m2);
+    // see if there is any alias for it
+    for (ln2=lfirst(d->mem_alias); ln2; ln2=lnext(ln2)) {
+      AVRMEM_ALIAS *a = ldata(ln2);
+      if (a->aliased_mem == m) {
+        // yes, duplicate it
+        AVRMEM_ALIAS *a2 = avr_dup_memalias(a);
+        // ... adjust the pointer ...
+        a2->aliased_mem = m2;
+        // ... and add to new list
+        ladd(p->mem_alias, a2);
+      }
+    }
   }
 
   for (i = 0; i < AVR_OP_MAX; i++) {
@@ -495,6 +627,8 @@ void avr_free_part(AVRPART * d)
 int i;
 	ldestroy_cb(d->mem, (void(*)(void *))avr_free_mem);
 	d->mem = NULL;
+	ldestroy_cb(d->mem_alias, (void(*)(void *))avr_free_memalias);
+	d->mem_alias = NULL;
     for(i=0;i<sizeof(d->op)/sizeof(d->op[0]);i++)
     {
     	if (d->op[i] != NULL)
@@ -621,40 +755,33 @@ void avr_display(FILE * f, AVRPART * p, const char * prefix, int verbose)
   LNODEID ln;
   AVRMEM * m;
 
-  fprintf(f,
-          "%sAVR Part                      : %s\n"
-          "%sChip Erase delay              : %d us\n"
-          "%sPAGEL                         : P%02X\n"
-          "%sBS2                           : P%02X\n"
-          "%sRESET disposition             : %s\n"
-          "%sRETRY pulse                   : %s\n"
-          "%sserial program mode           : %s\n"
-          "%sparallel program mode         : %s\n"
-          "%sTimeout                       : %d\n"
-          "%sStabDelay                     : %d\n"
-          "%sCmdexeDelay                   : %d\n"
-          "%sSyncLoops                     : %d\n"
-          "%sByteDelay                     : %d\n"
-          "%sPollIndex                     : %d\n"
-          "%sPollValue                     : 0x%02x\n"
-          "%sMemory Detail                 :\n\n",
-          prefix, p->desc,
-          prefix, p->chip_erase_delay,
-          prefix, p->pagel,
-          prefix, p->bs2,
-          prefix, reset_disp_str(p->reset_disposition),
-          prefix, avr_pin_name(p->retry_pulse),
-          prefix, (p->flags & AVRPART_SERIALOK) ? "yes" : "no",
-          prefix, (p->flags & AVRPART_PARALLELOK) ?
-            ((p->flags & AVRPART_PSEUDOPARALLEL) ? "pseudo" : "yes") : "no",
-          prefix, p->timeout,
-          prefix, p->stabdelay,
-          prefix, p->cmdexedelay,
-          prefix, p->synchloops,
-          prefix, p->bytedelay,
-          prefix, p->pollindex,
-          prefix, p->pollvalue,
-          prefix);
+  fprintf(  f, "%sAVR Part                      : %s\n", prefix, p->desc);
+  if (p->chip_erase_delay)
+    fprintf(f, "%sChip Erase delay              : %d us\n", prefix, p->chip_erase_delay);
+  if (p->pagel)
+    fprintf(f, "%sPAGEL                         : P%02X\n", prefix, p->pagel);
+  if (p->bs2)
+    fprintf(f, "%sBS2                           : P%02X\n", prefix, p->bs2);
+  fprintf(  f, "%sRESET disposition             : %s\n", prefix, reset_disp_str(p->reset_disposition));
+  fprintf(  f, "%sRETRY pulse                   : %s\n", prefix, avr_pin_name(p->retry_pulse));
+  fprintf(  f, "%sSerial program mode           : %s\n", prefix, (p->flags & AVRPART_SERIALOK) ? "yes" : "no");
+  fprintf(  f, "%sParallel program mode         : %s\n", prefix, (p->flags & AVRPART_PARALLELOK) ?
+         ((p->flags & AVRPART_PSEUDOPARALLEL) ? "pseudo" : "yes") : "no");
+  if(p->timeout)
+    fprintf(f, "%sTimeout                       : %d\n", prefix, p->timeout);
+  if(p->stabdelay)
+    fprintf(f, "%sStabDelay                     : %d\n", prefix, p->stabdelay);
+  if(p->cmdexedelay)
+    fprintf(f, "%sCmdexeDelay                   : %d\n", prefix, p->cmdexedelay);
+  if(p->synchloops)
+    fprintf(f, "%sSyncLoops                     : %d\n", prefix, p->synchloops);
+  if(p->bytedelay)
+    fprintf(f, "%sByteDelay                     : %d\n", prefix, p->bytedelay);
+  if(p->pollindex)
+    fprintf(f, "%sPollIndex                     : %d\n", prefix, p->pollindex);
+  if(p->pollvalue)
+    fprintf(f, "%sPollValue                     : 0x%02x\n", prefix, p->pollvalue);
+  fprintf(  f, "%sMemory Detail                 :\n\n", prefix);
 
   px = prefix;
   i = strlen(prefix) + 5;
@@ -669,11 +796,11 @@ void avr_display(FILE * f, AVRPART * p, const char * prefix, int verbose)
   }
 
   if (verbose <= 2) {
-    avr_mem_display(px, f, NULL, 0, verbose);
+    avr_mem_display(px, f, NULL, p, 0, verbose);
   }
   for (ln=lfirst(p->mem); ln; ln=lnext(ln)) {
     m = ldata(ln);
-    avr_mem_display(px, f, m, i, verbose);
+    avr_mem_display(px, f, m, p, i, verbose);
   }
 
   if (buf)
