@@ -16,7 +16,7 @@
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: dfu.h 1268 2014-01-17 14:00:01Z joerg_wunsch $ */
+/* $Id$ */
 
 #ifndef dfu_h
 #define dfu_h
@@ -61,7 +61,7 @@ struct dfu_dev
 #else
 
 struct dfu_dev {
-  // empty
+  int dummy;
 };
 
 #endif
