@@ -16,10 +16,11 @@
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: flip2.c 1321 2014-06-13 20:07:40Z awachtler $ */
+/* $Id$ */
 
 #include "ac_cfg.h"
 
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -27,12 +28,6 @@
 #include <limits.h>
 #include <unistd.h>
 
-#if HAVE_STDINT_H
-#include <stdint.h>
-#elif HAVE_INTTYPES_H
-#include <inttypes.h>
-#endif
-
 #include "avrdude.h"
 #include "libavrdude.h"
 
@@ -215,7 +210,7 @@ int flip2_initialize(PROGRAMMER* pgm, AVRPART *part)
   /* A note about return values. Negative return values from this function are
    * interpreted as failure by main(), from where this function is called.
    * However such failures are interpreted as a device signature check failure
-   * and the user is adviced to use the -F option to override this check. In
+   * and the user is advised to use the -F option to override this check. In
    * our case, this is misleading, so we defer reporting an error until another
    * function is called. Thus, we always return 0 (success) from initialize().
    * I don't like this, but I don't want to mess with main().
@@ -617,7 +612,7 @@ int flip2_read_memory(struct dfu_dev *dfu,
       return -1;
     }
 
-    ptr += read_size;
+    ptr = (char*)ptr + read_size;
     addr += read_size;
     size -= read_size;
   }
@@ -680,7 +675,7 @@ int flip2_write_memory(struct dfu_dev *dfu,
       return -1;
     }
 
-    ptr += write_size;
+    ptr = (const char*)ptr + write_size;
     addr += write_size;
     size -= write_size;
   }
