@@ -16,7 +16,7 @@
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: jtagmkI.c 1345 2014-11-23 20:57:17Z joerg_wunsch $ */
+/* $Id$ */
 
 /*
  * avrdude interface for Atmel JTAG ICE (mkI) programmer
@@ -214,6 +214,7 @@ static int jtagmkI_send(PROGRAMMER * pgm, unsigned char * data, size_t len)
   if (serial_send(&pgm->fd, buf, len + 2) != 0) {
     avrdude_message(MSG_INFO, "%s: jtagmkI_send(): failed to send command to serial port\n",
                     progname);
+    free(buf);
     return -1;
   }
 
@@ -553,7 +554,7 @@ static int jtagmkI_initialize(PROGRAMMER * pgm, AVRPART * p)
                 progname, pgm->baudrate);
       if (jtagmkI_setparm(pgm, PARM_BITRATE, b) == 0) {
         PDATA(pgm)->initial_baudrate = pgm->baudrate; /* don't adjust again later */
-        serial_setspeed(&pgm->fd, pgm->baudrate);
+        serial_setparams(&pgm->fd, pgm->baudrate, SERIAL_8N1);
       }
     }
   }
@@ -648,9 +649,10 @@ static int jtagmkI_open(PROGRAMMER * pgm, char * port)
 
   for (i = 0; i < sizeof(baudtab) / sizeof(baudtab[0]); i++) {
     union pinfo pinfo;
-    pinfo.baud = baudtab[i].baud;
+    pinfo.serialinfo.baud = baudtab[i].baud;
+    pinfo.serialinfo.cflags = SERIAL_8N1;
     avrdude_message(MSG_NOTICE2, "%s: jtagmkI_open(): trying to sync at baud rate %ld:\n",
-                      progname, pinfo.baud);
+                      progname, pinfo.serialinfo.baud);
     if (serial_open(port, pinfo, &pgm->fd)==-1) {
       return -1;
     }
@@ -697,7 +699,7 @@ static void jtagmkI_close(PROGRAMMER * pgm)
                 "trying to set baudrate to %d\n",
                 progname, PDATA(pgm)->initial_baudrate);
       if (jtagmkI_setparm(pgm, PARM_BITRATE, b) == 0) {
-        serial_setspeed(&pgm->fd, pgm->baudrate);
+        serial_setparams(&pgm->fd, pgm->baudrate, SERIAL_8N1);
       }
     }
   }
@@ -1273,8 +1275,8 @@ static void jtagmkI_display(PROGRAMMER * pgm, const char * p)
       jtagmkI_getparm(pgm, PARM_SW_VERSION, &fw) < 0)
     return;
 
-  avrdude_message(MSG_INFO, "%sICE hardware version: 0x%02x\n", p, hw);
-  avrdude_message(MSG_INFO, "%sICE firmware version: 0x%02x\n", p, fw);
+  avrdude_message(MSG_INFO, "%sICE HW version: 0x%02x\n", p, hw);
+  avrdude_message(MSG_INFO, "%sICE FW version: 0x%02x\n", p, fw);
 
   jtagmkI_print_parms1(pgm, p);
 
@@ -1318,9 +1320,9 @@ static void jtagmkI_print_parms1(PROGRAMMER * pgm, const char * p)
     clk = 1e6;
   }
 
-  avrdude_message(MSG_INFO, "%sVtarget         : %.1f V\n", p,
+  avrdude_message(MSG_INFO, "%sVtarget       : %.1f V\n", p,
 	  6.25 * (unsigned)vtarget / 255.0);
-  avrdude_message(MSG_INFO, "%sJTAG clock      : %s (%.1f us)\n", p, clkstr,
+  avrdude_message(MSG_INFO, "%sJTAG clock    : %s (%.1f us)\n", p, clkstr,
 	  1.0e6 / clk);
 
   return;
