@@ -17,7 +17,7 @@
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: main.c 1523 2021-12-16 21:02:35Z joerg_wunsch $ */
+/* $Id$ */
 
 /*
  * Code to program an Atmel AVR device through one of the supported
@@ -33,6 +33,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <whereami.h>
 #include <stdarg.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -121,9 +122,6 @@ static void usage(void)
  "                             is performed in the order specified.\n"
  "  -n                         Do not write anything to the device.\n"
  "  -V                         Do not verify.\n"
- "  -u                         Disable safemode, default when running from a script.\n"
- "  -s                         Silent safemode operation, will not ask you if\n"
- "                             fuses should be changed back.\n"
  "  -t                         Enter terminal mode.\n"
  "  -E <exitspec>[,<exitspec>] List programmer exit specifications.\n"
  "  -x <extended_param>        Pass <extended_param> to programmer.\n"
@@ -131,7 +129,7 @@ static void usage(void)
  "  -q                         Quell progress output. -q -q for less.\n"
  "  -l logfile                 Use logfile rather than stderr for diagnostics.\n"
  "  -?                         Display this usage.\n"
- "\navrdude version %s, URL: <http://savannah.nongnu.org/projects/avrdude/>\n"
+ "\navrdude version %s, URL: <https://github.com/avrdudes/avrdude>\n"
           ,progname, version);
 }
 
@@ -301,6 +299,17 @@ static void cleanup_main(void)
     cleanup_config();
 }
 
+static void replace_backslashes(char *s)
+{
+  // Replace all backslashes with forward slashes
+  for (int i = 0; i < strlen(s); i++) {
+    if (s[i] == '\\') {
+      s[i] = '/';
+    }
+  }
+}
+
+
 /*
  * main routine
  */
@@ -329,27 +338,27 @@ int main(int argc, char * argv [])
   char  * partdesc;    /* part id */
   char    sys_config[PATH_MAX]; /* system wide config file */
   char    usr_config[PATH_MAX]; /* per-user config file */
+  char    executable_abspath[PATH_MAX]; /* absolute path to avrdude executable */
+  char    executable_dirpath[PATH_MAX]; /* absolute path to folder with executable */
+  bool    executable_abspath_found = false; /* absolute path to executable found */
+  bool    sys_config_found = false; /* 'avrdude.conf' file found */
   char  * e;           /* for strtol() error checking */
   int     baudrate;    /* override default programmer baud rate */
   double  bitclock;    /* Specify programmer bit clock (JTAG ICE) */
   int     ispdelay;    /* Specify the delay for ISP clock */
-  int     safemode;    /* Enable safemode, 1=safemode on, 0=normal */
-  int     silentsafe;  /* Don't ask about fuses, 1=silent, 0=normal */
   int     init_ok;     /* Device initialization worked well */
   int     is_open;     /* Device open succeeded */
   char  * logfile;     /* Use logfile rather than stderr for diagnostics */
   enum updateflags uflags = UF_AUTO_ERASE; /* Flags for do_op() */
-  unsigned char safemode_lfuse = 0xff;
-  unsigned char safemode_hfuse = 0xff;
-  unsigned char safemode_efuse = 0xff;
-  unsigned char safemode_fuse = 0xff;
 
-  char * safemode_response;
-  int fuses_updated = 0;
-#if !defined(WIN32NATIVE)
+#if !defined(WIN32)
   char  * homedir;
 #endif
 
+#ifdef _MSC_VER
+  _set_printf_count_output(1);
+#endif
+
   /*
    * Set line buffering for file descriptors so we see stdout and stderr
    * properly interleaved.
@@ -357,12 +366,14 @@ int main(int argc, char * argv [])
   setvbuf(stdout, (char*)NULL, _IOLBF, 0);
   setvbuf(stderr, (char*)NULL, _IOLBF, 0);
 
+  sys_config[0] = '\0';
+
   progname = strrchr(argv[0],'/');
 
-#if defined (WIN32NATIVE)
+#if defined (WIN32)
   /* take care of backslash as dir sep in W32 */
   if (!progname) progname = strrchr(argv[0],'\\');
-#endif /* WIN32NATIVE */
+#endif /* WIN32 */
 
   if (progname)
     progname++;
@@ -372,7 +383,6 @@ int main(int argc, char * argv [])
   default_parallel[0] = 0;
   default_serial[0]   = 0;
   default_bitclock    = 0.0;
-  default_safemode    = -1;
 
   init_config();
 
@@ -412,36 +422,9 @@ int main(int argc, char * argv [])
   baudrate      = 0;
   bitclock      = 0.0;
   ispdelay      = 0;
-  safemode      = 1;       /* Safemode on by default */
-  silentsafe    = 0;       /* Ask by default */
   is_open       = 0;
   logfile       = NULL;
 
-#if defined(WIN32NATIVE)
-
-  win_sys_config_set(sys_config);
-  win_usr_config_set(usr_config);
-
-#else
-
-  strcpy(sys_config, CONFIG_DIR);
-  i = strlen(sys_config);
-  if (i && (sys_config[i-1] != '/'))
-    strcat(sys_config, "/");
-  strcat(sys_config, "avrdude.conf");
-
-  usr_config[0] = 0;
-  homedir = getenv("HOME");
-  if (homedir != NULL) {
-    strcpy(usr_config, homedir);
-    i = strlen(usr_config);
-    if (i && (usr_config[i-1] != '/'))
-      strcat(usr_config, "/");
-    strcat(usr_config, ".avrduderc");
-  }
-
-#endif
-
   len = strlen(progname) + 2;
   for (i=0; i<len; i++)
     progbuf[i] = ' ';
@@ -584,17 +567,13 @@ int main(int argc, char * argv [])
         quell_progress++ ;
         break;
 
-      case 's' : /* Silent safemode */
-        silentsafe = 1;
-        safemode = 1;
-        break;
-        
       case 't': /* enter terminal mode */
         terminal = 1;
         break;
 
-      case 'u' : /* Disable safemode */
-        safemode = 0;
+      case 'u':
+        avrdude_message(MSG_INFO, "%s: \"safemode\" feature no longer supported\n",
+                progname);
         break;
 
       case 'U':
@@ -659,6 +638,122 @@ int main(int argc, char * argv [])
     }
   }
 
+  /* search for system configuration file unless -C conffile was given */
+  if (strlen(sys_config) == 0) {
+    /*
+     * EXECUTABLE ABSPATH
+     * ------------------
+     * Determine the absolute path to avrdude executable. This will be used to
+     * locate the 'avrdude.conf' file later.
+     */
+    int executable_dirpath_len;
+    int executable_abspath_len = wai_getExecutablePath(
+      executable_abspath,
+      PATH_MAX,
+      &executable_dirpath_len
+    );
+    if (
+        (executable_abspath_len != -1) &&
+        (executable_abspath_len != 0) &&
+        (executable_dirpath_len != -1) &&
+        (executable_dirpath_len != 0)
+        ) {
+      // All requirements satisfied, executable path was found
+      executable_abspath_found = true;
+
+      // Make sure the string is null terminated
+      executable_abspath[executable_abspath_len] = '\0';
+
+      replace_backslashes(executable_abspath);
+
+      // Define 'executable_dirpath' to be the path to the parent folder of the
+      // executable.
+      strcpy(executable_dirpath, executable_abspath);
+      executable_dirpath[executable_dirpath_len] = '\0';
+
+      // Debug output
+      avrdude_message(MSG_DEBUG, "executable_abspath = %s\n", executable_abspath);
+      avrdude_message(MSG_DEBUG, "executable_abspath_len = %i\n", executable_abspath_len);
+      avrdude_message(MSG_DEBUG, "executable_dirpath = %s\n", executable_dirpath);
+      avrdude_message(MSG_DEBUG, "executable_dirpath_len = %i\n", executable_dirpath_len);
+    }
+
+    /*
+     * SYSTEM CONFIG
+     * -------------
+     * Determine the location of 'avrdude.conf'. Check in this order:
+     *  1. <dirpath of executable>/../etc/avrdude.conf
+     *  2. <dirpath of executable>/avrdude.conf
+     *  3. CONFIG_DIR/avrdude.conf
+     *
+     * When found, write the result into the 'sys_config' variable.
+     */
+    if (executable_abspath_found) {
+      // 1. Check <dirpath of executable>/../etc/avrdude.conf
+      strcpy(sys_config, executable_dirpath);
+      sys_config[PATH_MAX - 1] = '\0';
+      i = strlen(sys_config);
+      if (i && (sys_config[i - 1] != '/'))
+        strcat(sys_config, "/");
+      strcat(sys_config, "../etc/" SYSTEM_CONF_FILE);
+      sys_config[PATH_MAX - 1] = '\0';
+      if (access(sys_config, F_OK) == 0) {
+        sys_config_found = true;
+      }
+      else {
+        // 2. Check <dirpath of executable>/avrdude.conf
+        strcpy(sys_config, executable_dirpath);
+        sys_config[PATH_MAX - 1] = '\0';
+        i = strlen(sys_config);
+        if (i && (sys_config[i - 1] != '/'))
+          strcat(sys_config, "/");
+        strcat(sys_config, SYSTEM_CONF_FILE);
+        sys_config[PATH_MAX - 1] = '\0';
+        if (access(sys_config, F_OK) == 0) {
+          sys_config_found = true;
+        }
+      }
+    }
+    if (!sys_config_found) {
+      // 3. Check CONFIG_DIR/avrdude.conf
+#if defined(WIN32)
+      win_sys_config_set(sys_config);
+#else
+      strcpy(sys_config, CONFIG_DIR);
+      i = strlen(sys_config);
+      if (i && (sys_config[i - 1] != '/'))
+        strcat(sys_config, "/");
+      strcat(sys_config, SYSTEM_CONF_FILE);
+#endif
+      if (access(sys_config, F_OK) == 0) {
+        sys_config_found = true;
+      }
+    }
+  }
+  // Debug output
+  avrdude_message(MSG_DEBUG, "sys_config = %s\n", sys_config);
+  avrdude_message(MSG_DEBUG, "sys_config_found = %s\n", sys_config_found ? "true" : "false");
+  avrdude_message(MSG_DEBUG, "\n");
+
+  /*
+   * USER CONFIG
+   * -----------
+   * Determine the location of '.avrduderc'.
+   */
+#if defined(WIN32)
+  win_usr_config_set(usr_config);
+#else
+  usr_config[0] = 0;
+  homedir = getenv("HOME");
+  if (homedir != NULL) {
+    strcpy(usr_config, homedir);
+    i = strlen(usr_config);
+    if (i && (usr_config[i - 1] != '/'))
+      strcat(usr_config, "/");
+    strcat(usr_config, USER_CONF_FILE);
+  }
+#endif
+
   if (quell_progress == 0) {
     if (isatty (STDERR_FILENO))
       update_progress = update_progress_tty;
@@ -862,29 +957,6 @@ int main(int argc, char * argv [])
     }
   }
 
-  if (default_safemode == 0) {
-    /* configuration disables safemode: revert meaning of -u */
-    if (safemode == 0)
-      /* -u was given: enable safemode */
-      safemode = 1;
-    else
-      /* -u not given: turn off */
-      safemode = 0;
-  }
-
-  if (isatty(STDIN_FILENO) == 0 && silentsafe == 0)
-    safemode  = 0;       /* Turn off safemode if this isn't a terminal */
-
-
-  if(p->flags & AVRPART_AVR32) {
-    safemode = 0;
-  }
-
-  if(p->flags & (AVRPART_HAS_PDI | AVRPART_HAS_TPI)) {
-    safemode = 0;
-  }
-
-
   if (avr_initmem(p) != 0)
   {
     avrdude_message(MSG_INFO, "\n%s: failed to initialize memories\n",
@@ -952,6 +1024,9 @@ int main(int argc, char * argv [])
 
   rc = pgm->open(pgm, port);
   if (rc < 0) {
+    avrdude_message(MSG_INFO,
+                    "%s: opening programmer \"%s\" on port \"%s\" failed\n",
+                    progname, programmer, port);
     exitrc = 1;
     pgm->ppidata = 0; /* clear all bits at exit */
     goto main_exit;
@@ -1004,7 +1079,7 @@ int main(int argc, char * argv [])
   pgm->vfy_led(pgm, OFF);
 
   /*
-   * initialize the chip in preperation for accepting commands
+   * initialize the chip in preparation for accepting commands
    */
   init_ok = (rc = pgm->initialize(pgm, p)) >= 0;
   if (!init_ok) {
@@ -1109,12 +1184,19 @@ int main(int argc, char * argv [])
         if (sig->buf[i] != 0x00)
           zz = 0;
       }
+
+      bool signature_matches =
+          sig->size == 3 &&
+          sig->buf[0] == p->signature[0] &&
+          sig->buf[1] == p->signature[1] &&
+          sig->buf[2] == p->signature[2];
+
       if (quell_progress < 2) {
         AVRPART * part;
 
         part = locate_part_by_signature(part_list, sig->buf, sig->size);
         if (part) {
-          avrdude_message(MSG_INFO, " (probably %s)", part->id);
+          avrdude_message(MSG_INFO, " (probably %s)", signature_matches ? p->id : part->id);
         }
       }
       if (ff || zz) {
@@ -1143,10 +1225,7 @@ int main(int argc, char * argv [])
         }
       }
 
-      if (sig->size != 3 ||
-          sig->buf[0] != p->signature[0] ||
-          sig->buf[1] != p->signature[1] ||
-          sig->buf[2] != p->signature[2]) {
+      if (!signature_matches) {
         avrdude_message(MSG_INFO, "%s: Expected signature for %s is %02X %02X %02X\n",
                         progname, p->desc,
                         p->signature[0], p->signature[1], p->signature[2]);
@@ -1161,36 +1240,6 @@ int main(int argc, char * argv [])
     }
   }
 
-  if (init_ok && safemode == 1) {
-    /* If safemode is enabled, go ahead and read the current low, high,
-       and extended fuse bytes as needed */
-
-    rc = safemode_readfuses(&safemode_lfuse, &safemode_hfuse,
-                           &safemode_efuse, &safemode_fuse, pgm, p);
-
-    if (rc != 0) {
-
-	  //Check if the programmer just doesn't support reading
-  	  if (rc == -5)
-			{
-				avrdude_message(MSG_NOTICE, "%s: safemode: Fuse reading not supported by programmer.\n"
-						"              Safemode disabled.\n", progname);
-			}
-      else
-			{
-
-      		avrdude_message(MSG_INFO, "%s: safemode: To protect your AVR the programming "
-            				    "will be aborted\n",
-               					 progname);
-      		exitrc = 1;
-		    goto main_exit;
-			}
-    } else {
-      //Save the fuses as default
-      safemode_memfuses(1, &safemode_lfuse, &safemode_hfuse, &safemode_efuse, &safemode_fuse);
-    }
-  }
-
   if (uflags & UF_AUTO_ERASE) {
     if ((p->flags & AVRPART_HAS_PDI) && pgm->page_erase != NULL &&
         lsize(updates) > 0) {
@@ -1266,169 +1315,6 @@ int main(int argc, char * argv [])
     }
   }
 
-  /* Right before we exit programming mode, which will make the fuse
-     bits active, check to make sure they are still correct */
-  if (safemode == 1) {
-    /* If safemode is enabled, go ahead and read the current low,
-     * high, and extended fuse bytes as needed */
-    unsigned char safemodeafter_lfuse = 0xff;
-    unsigned char safemodeafter_hfuse = 0xff;
-    unsigned char safemodeafter_efuse = 0xff;
-    unsigned char safemodeafter_fuse  = 0xff;
-    unsigned char failures = 0;
-    char yes[1] = {'y'};
-
-    if (quell_progress < 2) {
-      avrdude_message(MSG_INFO, "\n");
-    }
-
-    //Restore the default fuse values
-    safemode_memfuses(0, &safemode_lfuse, &safemode_hfuse, &safemode_efuse, &safemode_fuse);
-
-    /* Try reading back fuses, make sure they are reliable to read back */
-    if (safemode_readfuses(&safemodeafter_lfuse, &safemodeafter_hfuse,
-                           &safemodeafter_efuse, &safemodeafter_fuse, pgm, p) != 0) {
-      /* Uh-oh.. try once more to read back fuses */
-      if (safemode_readfuses(&safemodeafter_lfuse, &safemodeafter_hfuse,
-                             &safemodeafter_efuse, &safemodeafter_fuse, pgm, p) != 0) {
-        avrdude_message(MSG_INFO, "%s: safemode: Sorry, reading back fuses was unreliable. "
-                        "I have given up and exited programming mode\n",
-                        progname);
-        exitrc = 1;
-        goto main_exit;        
-      }
-    }
-    
-    AVRMEM * m;
-    
-    /* Now check what fuses are against what they should be */
-    m = avr_locate_mem(p, "fuse");
-    if (compare_memory_masked(m, safemodeafter_fuse, safemode_fuse)) {
-      fuses_updated = 1;
-      avrdude_message(MSG_INFO, "%s: safemode: fuse changed! Was %x, and is now %x\n",
-              progname, safemode_fuse, safemodeafter_fuse);
-
-              
-      /* Ask user - should we change them */
-       
-       if (silentsafe == 0)
-            safemode_response = terminal_get_input("Would you like this fuse to be changed back? [y/n] ");
-       else
-            safemode_response = yes;
-       
-       if (tolower((int)(safemode_response[0])) == 'y') {
-              
-            /* Enough chit-chat, time to program some fuses and check them */
-            if (safemode_writefuse (safemode_fuse, "fuse", pgm, p,
-                                    10) == 0) {
-                avrdude_message(MSG_INFO, "%s: safemode: and is now rescued\n", progname);
-            }
-            else {
-                avrdude_message(MSG_INFO, "%s: and COULD NOT be changed\n", progname);
-                failures++;
-            }
-      }
-    }
-
-    /* Now check what fuses are against what they should be */
-    m = avr_locate_mem(p, "lfuse");
-    if (compare_memory_masked(m, safemodeafter_lfuse, safemode_lfuse)) {
-      fuses_updated = 1;
-      avrdude_message(MSG_INFO, "%s: safemode: lfuse changed! Was %x, and is now %x\n",
-              progname, safemode_lfuse, safemodeafter_lfuse);
-
-              
-      /* Ask user - should we change them */
-       
-       if (silentsafe == 0)
-            safemode_response = terminal_get_input("Would you like this fuse to be changed back? [y/n] ");
-       else
-            safemode_response = yes;
-       
-       if (tolower((int)(safemode_response[0])) == 'y') {
-              
-            /* Enough chit-chat, time to program some fuses and check them */
-            if (safemode_writefuse (safemode_lfuse, "lfuse", pgm, p,
-                                    10) == 0) {
-                avrdude_message(MSG_INFO, "%s: safemode: and is now rescued\n", progname);
-            }
-            else {
-                avrdude_message(MSG_INFO, "%s: and COULD NOT be changed\n", progname);
-                failures++;
-            }
-      }
-    }
-
-    /* Now check what fuses are against what they should be */
-    m = avr_locate_mem(p, "hfuse");
-    if (compare_memory_masked(m, safemodeafter_hfuse, safemode_hfuse)) {
-      fuses_updated = 1;
-      avrdude_message(MSG_INFO, "%s: safemode: hfuse changed! Was %x, and is now %x\n",
-              progname, safemode_hfuse, safemodeafter_hfuse);
-              
-      /* Ask user - should we change them */
-       if (silentsafe == 0)
-            safemode_response = terminal_get_input("Would you like this fuse to be changed back? [y/n] ");
-       else
-            safemode_response = yes;
-       if (tolower((int)(safemode_response[0])) == 'y') {
-
-            /* Enough chit-chat, time to program some fuses and check them */
-            if (safemode_writefuse(safemode_hfuse, "hfuse", pgm, p,
-                                    10) == 0) {
-                avrdude_message(MSG_INFO, "%s: safemode: and is now rescued\n", progname);
-            }
-            else {
-                avrdude_message(MSG_INFO, "%s: and COULD NOT be changed\n", progname);
-                failures++;
-            }
-      }
-    }
-
-    /* Now check what fuses are against what they should be */
-    m = avr_locate_mem(p, "efuse");
-    if (compare_memory_masked(m, safemodeafter_efuse, safemode_efuse)) {
-      fuses_updated = 1;
-      avrdude_message(MSG_INFO, "%s: safemode: efuse changed! Was %x, and is now %x\n",
-              progname, safemode_efuse, safemodeafter_efuse);
-
-      /* Ask user - should we change them */
-       if (silentsafe == 0)
-            safemode_response = terminal_get_input("Would you like this fuse to be changed back? [y/n] ");
-       else
-            safemode_response = yes;
-       if (tolower((int)(safemode_response[0])) == 'y') {
-              
-            /* Enough chit-chat, time to program some fuses and check them */
-            if (safemode_writefuse (safemode_efuse, "efuse", pgm, p,
-                                    10) == 0) {
-                avrdude_message(MSG_INFO, "%s: safemode: and is now rescued\n", progname);
-            }
-            else {
-                avrdude_message(MSG_INFO, "%s: and COULD NOT be changed\n", progname);
-                failures++;
-            }
-       }
-    }
-
-    if (quell_progress < 2) {
-      avrdude_message(MSG_INFO, "%s: safemode: ", progname);
-      if (failures == 0) {
-        avrdude_message(MSG_INFO, "Fuses OK (E:%02X, H:%02X, L:%02X)\n",
-                safemodeafter_efuse, safemodeafter_hfuse, safemodeafter_lfuse);
-      }
-      else {
-        avrdude_message(MSG_INFO, "Fuses not recovered, sorry\n");
-      }
-    }
-
-    if (fuses_updated) {
-      exitrc = 1;
-    }
-
-  }
-
-
 main_exit:
 
   /*
