@@ -41,13 +41,16 @@
 #include "jtag3.h"
 #include "linuxgpio.h"
 #include "linuxspi.h"
+#include "micronucleus.h"
 #include "par.h"
 #include "pickit2.h"
 #include "ppi.h"
 #include "serbb.h"
+#include "serialupdi.h"
 #include "stk500.h"
 #include "stk500generic.h"
 #include "stk500v2.h"
+#include "teensy.h"
 #include "usbasp.h"
 #include "usbtiny.h"
 #include "wiring.h"
@@ -84,9 +87,11 @@ const PROGRAMMER_TYPE programmers_types[] = {
         {"jtagice3_isp", stk500v2_jtag3_initpgm, stk500v2_jtag3_desc},
         {"linuxgpio", linuxgpio_initpgm, linuxgpio_desc},
         {"linuxspi", linuxspi_initpgm, linuxspi_desc},
+        {"micronucleus", micronucleus_initpgm, micronucleus_desc},
         {"par", par_initpgm, par_desc},
         {"pickit2", pickit2_initpgm, pickit2_desc},
         {"serbb", serbb_initpgm, serbb_desc},
+        {"serialupdi", serialupdi_initpgm, serialupdi_desc},
         {"stk500", stk500_initpgm, stk500_desc},
         {"stk500generic", stk500generic_initpgm, stk500generic_desc},
         {"stk500v2", stk500v2_initpgm, stk500v2_desc},
@@ -95,6 +100,7 @@ const PROGRAMMER_TYPE programmers_types[] = {
         {"stk600", stk600_initpgm, stk600_desc},
         {"stk600hvsp", stk600hvsp_initpgm, stk600hvsp_desc},
         {"stk600pp", stk600pp_initpgm, stk600pp_desc},
+        {"teensy", teensy_initpgm, teensy_desc},
         {"usbasp", usbasp_initpgm, usbasp_desc},
         {"usbtiny", usbtiny_initpgm, usbtiny_desc},
         {"wiring", wiring_initpgm, wiring_desc},
