@@ -17,22 +17,18 @@
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: ser_win32.c 1499 2021-11-27 21:15:55Z joerg_wunsch $ */
-
 /*
  * Native Win32 serial interface for avrdude.
  */
 
 #include "ac_cfg.h"
 
-#if defined(WIN32NATIVE)
-
-#ifdef HAVE_LIBWS2_32
-/* winsock2.h must be included before windows.h from avrdude.h... */
-#  include <winsock2.h>
-#endif
+#if defined(WIN32)
 
+#define WIN32_LEAN_AND_MEAN
 #include <windows.h>
+#include <winsock2.h>
+#include <stdlib.h>
 #include <stdio.h>
 #include <ctype.h>   /* for isprint */
 #include <errno.h>   /* ENOTTY */
@@ -54,6 +50,8 @@ static unsigned char serial_over_ethernet = 0;
 /* HANDLE hComPort=INVALID_HANDLE_VALUE; */
 
 static struct baud_mapping baud_lookup_table [] = {
+  { 300,    CBR_300 },
+  { 600,    CBR_600 },
   { 1200,   CBR_1200 },
   { 2400,   CBR_2400 },
   { 4800,   CBR_4800 },
@@ -97,7 +95,7 @@ static BOOL serial_w32SetTimeOut(HANDLE hComPort, DWORD timeout) // in ms
 	return SetCommTimeouts(hComPort, &ctmo);
 }
 
-static int ser_setspeed(union filedescriptor *fd, long baud)
+static int ser_setparams(union filedescriptor *fd, long baud, unsigned long cflags)
 {
 	if (serial_over_ethernet) {
 		return -ENOTTY;
@@ -111,9 +109,39 @@ static int ser_setspeed(union filedescriptor *fd, long baud)
 		dcb.fBinary = 1;
 		dcb.fDtrControl = DTR_CONTROL_DISABLE;
 		dcb.fRtsControl = RTS_CONTROL_DISABLE;
-		dcb.ByteSize = 8;
-		dcb.Parity = NOPARITY;
-		dcb.StopBits = ONESTOPBIT;
+		switch ((cflags & (SERIAL_CS5 | SERIAL_CS6 | SERIAL_CS7 | SERIAL_CS8))) {
+			case SERIAL_CS5:
+				dcb.ByteSize = 5;
+				break;
+			case SERIAL_CS6:
+				dcb.ByteSize = 6;
+				break;
+			case SERIAL_CS7:
+				dcb.ByteSize = 7;
+				break;
+			case SERIAL_CS8:
+				dcb.ByteSize = 8;
+				break;
+		}
+		switch ((cflags & (SERIAL_NO_PARITY | SERIAL_PARENB | SERIAL_PARODD))) {
+			case SERIAL_NO_PARITY:
+				dcb.Parity = NOPARITY;
+				break;
+			case SERIAL_PARENB:
+				dcb.Parity = EVENPARITY;
+				break;
+			case SERIAL_PARODD:
+				dcb.Parity = ODDPARITY;
+				break;
+		}
+		switch ((cflags & (SERIAL_NO_CSTOPB | SERIAL_CSTOPB))) {
+			case SERIAL_NO_CSTOPB:
+				dcb.StopBits = ONESTOPBIT;
+				break;
+			case SERIAL_CSTOPB:
+				dcb.StopBits = TWOSTOPBITS;
+				break;
+		}
 
 		if (!SetCommState(hComPort, &dcb))
 			return -1;
@@ -122,7 +150,6 @@ static int ser_setspeed(union filedescriptor *fd, long baud)
 	}
 }
 
-#ifdef HAVE_LIBWS2_32
 static int
 net_open(const char *port, union filedescriptor *fdp)
 {
@@ -214,7 +241,6 @@ net_open(const char *port, union filedescriptor *fdp)
 	serial_over_ethernet = 1;
 	return 0;
 }
-#endif
 
 
 static int ser_open(char * port, union pinfo pinfo, union filedescriptor *fdp)
@@ -228,14 +254,7 @@ static int ser_open(char * port, union pinfo pinfo, union filedescriptor *fdp)
 	 * handle it as a TCP connection to a terminal server.
 	 */
 	if (strncmp(port, "net:", strlen("net:")) == 0) {
-#ifdef HAVE_LIBWS2_32
 		return net_open(port + strlen("net:"), fdp);
-#else
-		avrdude_message(MSG_INFO, "%s: ser_open(): "
-				"not configured for TCP connections\n",
-                                progname);
-		return -1;
-#endif
 	}
 
 	if (strncasecmp(port, "com", strlen("com")) == 0) {
@@ -283,7 +302,7 @@ static int ser_open(char * port, union pinfo pinfo, union filedescriptor *fdp)
 	}
 
         fdp->pfd = (void *)hComPort;
-	if (ser_setspeed(fdp, pinfo.baud) != 0)
+	if (ser_setparams(fdp, pinfo.serialinfo.baud, pinfo.serialinfo.cflags) != 0)
 	{
 		CloseHandle(hComPort);
 		avrdude_message(MSG_INFO, "%s: ser_open(): can't set com-state for \"%s\"\n",
@@ -338,7 +357,6 @@ static int ser_set_dtr_rts(union filedescriptor *fd, int is_on)
 	}
 }
 
-#ifdef HAVE_LIBWS2_32
 static int net_send(union filedescriptor *fd, const unsigned char * buf, size_t buflen)
 {
 	LPVOID lpMsgBuf;
@@ -397,16 +415,13 @@ static int net_send(union filedescriptor *fd, const unsigned char * buf, size_t
 
 	return 0;
 }
-#endif
 
 
 static int ser_send(union filedescriptor *fd, const unsigned char * buf, size_t buflen)
 {
-#ifdef HAVE_LIBWS2_32
 	if (serial_over_ethernet) {
 		return net_send(fd, buf, buflen);
 	}
-#endif
 
 	size_t len = buflen;
 	unsigned char c='\0';
@@ -461,7 +476,6 @@ static int ser_send(union filedescriptor *fd, const unsigned char * buf, size_t
 }
 
 
-#ifdef HAVE_LIBWS2_32
 static int net_recv(union filedescriptor *fd, unsigned char * buf, size_t buflen)
 {
 	LPVOID lpMsgBuf;
@@ -555,15 +569,12 @@ reselect:
 
 	return 0;
 }
-#endif
 
 static int ser_recv(union filedescriptor *fd, unsigned char * buf, size_t buflen)
 {
-#ifdef HAVE_LIBWS2_32
 	if (serial_over_ethernet) {
 		return net_recv(fd, buf, buflen);
 	}
-#endif
 
 	unsigned char c;
 	unsigned char * p = buf;
@@ -628,7 +639,6 @@ static int ser_recv(union filedescriptor *fd, unsigned char * buf, size_t buflen
   return 0;
 }
 
-#ifdef HAVE_LIBWS2_32
 static int net_drain(union filedescriptor *fd, int display)
 {
 	LPVOID lpMsgBuf;
@@ -707,15 +717,12 @@ static int net_drain(union filedescriptor *fd, int display)
 
 	return 0;
 }
-#endif
 
 static int ser_drain(union filedescriptor *fd, int display)
 {
-#ifdef HAVE_LIBWS2_32
 	if (serial_over_ethernet) {
 		return net_drain(fd, display);
 	}
-#endif
 
 	// int rc;
 	unsigned char buf[10];
@@ -770,7 +777,7 @@ static int ser_drain(union filedescriptor *fd, int display)
 struct serial_device serial_serdev =
 {
   .open = ser_open,
-  .setspeed = ser_setspeed,
+  .setparams = ser_setparams,
   .close = ser_close,
   .send = ser_send,
   .recv = ser_recv,
@@ -781,4 +788,4 @@ struct serial_device serial_serdev =
 
 struct serial_device *serdev = &serial_serdev;
 
-#endif /* WIN32NATIVE */
+#endif /* WIN32 */
