@@ -1,7 +1,7 @@
 /*
  * avrdude - A Downloader/Uploader for AVR device programmers
  * Copyright (C) 2000, 2001, 2002, 2003  Brian S. Dean <bsd@bsdhome.com>
- * Copyright (C) 2005 Michael Holzt <kju-avr@fqdn.org>
+ * Copyright (C) 2005 Juliane Holzt <avrdude@juliane.holzt.de>
  * Copyright (C) 2006 Joerg Wunsch <j@uriah.heep.sax.de>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -17,13 +17,13 @@
  * You should have received a copy of the GNU General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
-/* $Id: serbb_posix.c 1321 2014-06-13 20:07:40Z awachtler $ */
+/* $Id$ */
 
 /*
  * Posix serial bitbanging interface for avrdude.
  */
 
-#if !defined(WIN32NATIVE)
+#if !defined(WIN32)
 
 #include "ac_cfg.h"
 
@@ -318,4 +318,4 @@ void serbb_initpgm(PROGRAMMER *pgm)
   pgm->write_byte     = avr_write_byte_default;
 }
 
-#endif  /* WIN32NATIVE */
+#endif  /* WIN32 */
