@@ -16,22 +16,21 @@
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: term.c 1471 2021-11-12 22:11:49Z joerg_wunsch $ */
+/* $Id$ */
 
 #include "ac_cfg.h"
 
 #include <ctype.h>
 #include <string.h>
 #include <stdio.h>
+#include <stdint.h>
 #include <stdlib.h>
 #include <limits.h>
 
 #if defined(HAVE_LIBREADLINE)
-#if !defined(WIN32NATIVE)
 #  include <readline/readline.h>
 #  include <readline/history.h>
 #endif
-#endif
 
 #include "avrdude.h"
 #include "term.h"
@@ -99,8 +98,8 @@ struct command cmd[] = {
   { "sig",   cmd_sig,   "display device signature bytes" },
   { "part",  cmd_part,  "display the current part information" },
   { "send",  cmd_send,  "send a raw command : %s <b1> <b2> <b3> <b4>" },
-  { "parms", cmd_parms, "display adjustable parameters (STK500 only)" },
-  { "vtarg", cmd_vtarg, "set <V[target]> (STK500 only)" },
+  { "parms", cmd_parms, "display adjustable parameters (STK500 and Curiosity Nano only)" },
+  { "vtarg", cmd_vtarg, "set <V[target]> (STK500 and Curiosity Nano only)" },
   { "varef", cmd_varef, "set <V[aref]> (STK500 only)" },
   { "fosc",  cmd_fosc,  "set <oscillator frequency> (STK500 only)" },
   { "sck",   cmd_sck,   "set <SCK period> (STK500 only)" },
@@ -127,9 +126,15 @@ static int nexttok(char * buf, char ** tok, char ** next)
     q++;
 
   /* isolate first token */
-  n = q+1;
-  while (*n && !isspace((int)*n))
+  n = q;
+  uint8_t quotes = 0;
+  while (*n && (!isspace((int)*n) || quotes)) {
+    if (*n == '\"')
+      quotes++;
+    else if (isspace((int)*n) && *(n-1) == '\"')
+      break;
     n++;
+  }
 
   if (*n) {
     *n = 0;
@@ -150,12 +155,10 @@ static int nexttok(char * buf, char ** tok, char ** next)
 static int hexdump_line(char * buffer, unsigned char * p, int n, int pad)
 {
   char * hexdata = "0123456789abcdef";
-  char * b;
-  int i, j;
+  char * b = buffer;
+  int32_t i = 0;
+  int32_t j = 0;
 
-  b = buffer;
-
-  j = 0;
   for (i=0; i<n; i++) {
     if (i && ((i % 8) == 0))
       b[j++] = ' ';
@@ -184,7 +187,7 @@ static int chardump_line(char * buffer, unsigned char * p, int n, int pad)
   int i;
   char b [ 128 ];
 
-  for (i=0; i<n; i++) {
+  for (int32_t i = 0; i < n; i++) {
     memcpy(b, p, n);
     buffer[i] = '.';
     if (isalpha((int)(b[i])) || isdigit((int)(b[i])) || ispunct((int)(b[i])))
@@ -193,7 +196,7 @@ static int chardump_line(char * buffer, unsigned char * p, int n, int pad)
       buffer[i] = ' ';
   }
 
-  for (i=n; i<pad; i++)
+  for (i = n; i < pad; i++)
     buffer[i] = ' ';
 
   buffer[i] = 0;
@@ -204,16 +207,13 @@ static int chardump_line(char * buffer, unsigned char * p, int n, int pad)
 
 static int hexdump_buf(FILE * f, int startaddr, unsigned char * buf, int len)
 {
-  int addr;
-  int n;
-  unsigned char * p;
   char dst1[80];
   char dst2[80];
 
-  addr = startaddr;
-  p = (unsigned char *)buf;
+  int32_t addr = startaddr;
+  unsigned char * p = (unsigned char *)buf;
   while (len) {
-    n = 16;
+    int32_t n = 16;
     if (n > len)
       n = len;
     hexdump_line(dst1, p, n, 48);
@@ -231,92 +231,99 @@ static int hexdump_buf(FILE * f, int startaddr, unsigned char * buf, int len)
 static int cmd_dump(PROGRAMMER * pgm, struct avrpart * p,
 		    int argc, char * argv[])
 {
-  static char prevmem[128] = {0};
-  char * e;
-  unsigned char * buf;
-  int maxsize;
-  unsigned long i;
-  static unsigned long addr=0;
-  static int len=64;
-  AVRMEM * mem;
-  char * memtype = NULL;
-  int rc;
-
-  if (!((argc == 2) || (argc == 4))) {
-    avrdude_message(MSG_INFO, "Usage: dump <memtype> [<addr> <len>]\n");
-    return -1;
+  if (argc < 2) {
+  avrdude_message(MSG_INFO, "Usage: %s <memtype> [<start addr> <len>]\n"
+                            "       %s <memtype> [<start addr> <...>]\n"
+                            "       %s <memtype> <...>\n"
+                            "       %s <memtype>\n",
+                            argv[0], argv[0], argv[0], argv[0]);
+  return -1;
   }
 
-  memtype = argv[1];
-
-  if (strncmp(prevmem, memtype, strlen(memtype)) != 0) {
-    addr = 0;
-    len  = 64;
-    strncpy(prevmem, memtype, sizeof(prevmem)-1);
-    prevmem[sizeof(prevmem)-1] = 0;
-  }
-
-  mem = avr_locate_mem(p, memtype);
+  enum { read_size = 256 };
+  static char prevmem[128] = {0x00};
+  char * memtype = argv[1];
+  AVRMEM * mem = avr_locate_mem(p, memtype);
   if (mem == NULL) {
     avrdude_message(MSG_INFO, "\"%s\" memory type not defined for part \"%s\"\n",
             memtype, p->desc);
     return -1;
   }
+  uint32_t maxsize = mem->size;
 
+  // Get start address if present
+  char * end_ptr;
+  static uint32_t addr = 0;
   if (argc == 4) {
-    addr = strtoul(argv[2], &e, 0);
-    if (*e || (e == argv[2])) {
-      avrdude_message(MSG_INFO, "%s (dump): can't parse address \"%s\"\n",
-              progname, argv[2]);
+    addr = strtoul(argv[2], &end_ptr, 0);
+    if (*end_ptr || (end_ptr == argv[2])) {
+      avrdude_message(MSG_INFO, "%s (%s): can't parse address \"%s\"\n",
+              progname, argv[0], argv[2]);
       return -1;
-    }
-
-    len = strtol(argv[3], &e, 0);
-    if (*e || (e == argv[3])) {
-      avrdude_message(MSG_INFO, "%s (dump): can't parse length \"%s\"\n",
-              progname, argv[3]);
+    } else if (addr >= maxsize) {
+      avrdude_message(MSG_INFO, "%s (%s): address 0x%05lx is out of range for %s memory\n",
+                      progname, argv[0], addr, mem->desc);
       return -1;
     }
   }
 
-  maxsize = mem->size;
-
-  if (addr >= maxsize) {
-    if (argc == 2) {
-      /* wrap around */
+  // Get no. bytes to read if present
+  static int32_t len = read_size;
+  if (argc >= 3) {
+    memset(prevmem, 0x00, sizeof(prevmem));
+    if (strcmp(argv[argc - 1], "...") == 0) {
+      if (argc == 3)
+        addr = 0;
+      len = maxsize - addr;
+    } else if (argc == 4) {
+      len = strtol(argv[3], &end_ptr, 0);
+      if (*end_ptr || (end_ptr == argv[3])) {
+        avrdude_message(MSG_INFO, "%s (%s): can't parse length \"%s\"\n",
+              progname, argv[0], argv[3]);
+        return -1;
+      }
+    } else {
+      len = read_size;
+    }
+  }
+  // No address or length specified
+  else if (argc == 2) {
+    if (strncmp(prevmem, memtype, strlen(memtype)) != 0) {
       addr = 0;
+      len  = read_size;
+      strncpy(prevmem, memtype, sizeof(prevmem) - 1);
+      prevmem[sizeof(prevmem) - 1] = 0;
     }
-    else {
-      avrdude_message(MSG_INFO, "%s (dump): address 0x%05lx is out of range for %s memory\n",
-                      progname, addr, mem->desc);
-      return -1;
-    }
+    if (addr >= maxsize)
+      addr = 0; // Wrap around
   }
 
-  /* trim len if nessary to not read past the end of memory */
+  // Trim len if nessary to not read past the end of memory
   if ((addr + len) > maxsize)
     len = maxsize - addr;
 
-  buf = malloc(len);
+  uint8_t * buf = malloc(len);
   if (buf == NULL) {
     avrdude_message(MSG_INFO, "%s (dump): out of memory\n", progname);
     return -1;
   }
 
-  for (i=0; i<len; i++) {
-    rc = pgm->read_byte(pgm, p, mem, addr+i, &buf[i]);
+  report_progress(0, 1, "Reading");
+  for (uint32_t i = 0; i < len; i++) {
+    int32_t rc = pgm->read_byte(pgm, p, mem, addr + i, &buf[i]);
     if (rc != 0) {
       avrdude_message(MSG_INFO, "error reading %s address 0x%05lx of part %s\n",
-              mem->desc, addr+i, p->desc);
+              mem->desc, addr + i, p->desc);
       if (rc == -1)
         avrdude_message(MSG_INFO, "read operation not supported on memory type \"%s\"\n",
                 mem->desc);
       return -1;
     }
+    report_progress(i, len, NULL);
   }
+  report_progress(1, 1, NULL);
 
   hexdump_buf(stdout, addr, buf, len);
-
   fprintf(stdout, "\n");
 
   free(buf);
@@ -330,35 +337,31 @@ static int cmd_dump(PROGRAMMER * pgm, struct avrpart * p,
 static int cmd_write(PROGRAMMER * pgm, struct avrpart * p,
 		     int argc, char * argv[])
 {
-  char * e;
-  int len, maxsize;
-  char * memtype;
-  unsigned long addr, i;
-  unsigned char * buf;
-  unsigned char b;
-  int rc;
-  int werror;
-  AVRMEM * mem;
-
   if (argc < 4) {
-    avrdude_message(MSG_INFO, "Usage: write <memtype> <addr> <byte1> "
-            "<byte2> ... <byteN>\n");
+    avrdude_message(MSG_INFO,
+      "Usage: write <memtype> <start addr> <data1> <data2> <dataN>\n"
+      "       write <memtype> <start addr> <no. bytes> <data1> <dataN> <...>\n\n"
+      "       Add a suffix to manually specify the size for each field:\n"
+      "       HH/hh: 8-bit, H/h/S/s: 16-bit, L/l: 32-bit, LL/ll: 64-bit, F/f: 32-bit float\n");
     return -1;
   }
 
-  memtype = argv[1];
-
-  mem = avr_locate_mem(p, memtype);
+  int32_t i;
+  uint8_t write_mode;       // Operation mode, "standard" or "fill"
+  uint8_t start_offset;     // Which argc argument
+  int32_t len;              // Number of bytes to write to memory
+  char * memtype = argv[1]; // Memory name string
+  AVRMEM * mem = avr_locate_mem(p, memtype);
   if (mem == NULL) {
     avrdude_message(MSG_INFO, "\"%s\" memory type not defined for part \"%s\"\n",
             memtype, p->desc);
     return -1;
   }
+  uint32_t maxsize = mem->size;
 
-  maxsize = mem->size;
-
-  addr = strtoul(argv[2], &e, 0);
-  if (*e || (e == argv[2])) {
+  char * end_ptr;
+  int32_t addr = strtoul(argv[2], &end_ptr, 0);
+  if (*end_ptr || (end_ptr == argv[2])) {
     avrdude_message(MSG_INFO, "%s (write): can't parse address \"%s\"\n",
             progname, argv[2]);
     return -1;
@@ -370,61 +373,228 @@ static int cmd_write(PROGRAMMER * pgm, struct avrpart * p,
     return -1;
   }
 
-  /* number of bytes to write at the specified address */
-  len = argc - 3;
-
-  if ((addr + len) > maxsize) {
-    avrdude_message(MSG_INFO, "%s (write): selected address and # bytes exceed "
-                    "range for %s memory\n",
-                    progname, memtype);
-    return -1;
-  }
-
-  buf = malloc(len);
+  // Allocate a buffer guaranteed to be large enough
+  uint8_t * buf = calloc(mem->size + 0x10 + strlen(argv[argc - 2]), sizeof(uint8_t));
   if (buf == NULL) {
     avrdude_message(MSG_INFO, "%s (write): out of memory\n", progname);
     return -1;
   }
 
-  for (i=3; i<argc; i++) {
-    buf[i-3] = strtoul(argv[i], &e, 0);
-    if (*e || (e == argv[i])) {
-      avrdude_message(MSG_INFO, "%s (write): can't parse byte \"%s\"\n",
-              progname, argv[i]);
+  // Find the first argument to write to flash and how many arguments to parse and write
+  if (strcmp(argv[argc - 1], "...") == 0) {
+    write_mode = WRITE_MODE_FILL;
+    start_offset = 4;
+    len = strtoul(argv[3], &end_ptr, 0);
+    if (*end_ptr || (end_ptr == argv[3])) {
+      avrdude_message(MSG_INFO, "%s (write ...): can't parse address \"%s\"\n",
+            progname, argv[3]);
       free(buf);
       return -1;
     }
+  } else {
+    write_mode = WRITE_MODE_STANDARD;
+    start_offset = 3;
+    len = argc - start_offset;
   }
 
-  pgm->err_led(pgm, OFF);
-  for (werror=0, i=0; i<len; i++) {
+  // Structure related to data that is being written to memory
+  struct Data {
+    // Data info
+    int32_t bytes_grown;
+    uint8_t size;
+    bool is_float;
+    bool is_signed;
+    char * str_ptr;
+    // Data union
+    union {
+      float f;
+      int64_t ll;
+      uint8_t a[8];
+    };
+  } data = {
+    .bytes_grown = 0,
+    .size        = 0,
+    .is_float    = false,
+    .is_signed   = false,
+    .str_ptr     = NULL,
+    .ll = 0
+  };
 
-    rc = avr_write_byte(pgm, p, mem, addr+i, buf[i]);
+  for (i = start_offset; i < len + start_offset; i++) {
+    data.is_float = false;
+    data.size = 0;
+
+    // Handle the next argument
+    if (i < argc - start_offset + 3) {
+      // Free string pointer if already allocated
+      if(data.str_ptr) {
+        free(data.str_ptr);
+        data.str_ptr = NULL;
+      }
+
+      // Get suffix if present
+      char suffix  = argv[i][strlen(argv[i]) - 1];
+      char lsuffix = argv[i][strlen(argv[i]) - 2];
+      if ((suffix == 'L' && lsuffix == 'L') || (suffix == 'l' && lsuffix == 'l')) {
+        argv[i][strlen(argv[i]) - 2] = '\0';
+        data.size = 8;
+      } else if (suffix == 'L' || suffix == 'l') {
+        argv[i][strlen(argv[i]) - 1] = '\0';
+        data.size = 4;
+      } else if ((suffix == 'F' || suffix == 'f') &&
+          strncmp(argv[i], "0x", 2) != 0 && strncmp(argv[i], "-0x", 3) != 0) {
+        argv[i][strlen(argv[i]) - 1] = '\0';
+        data.size = 4;
+      } else if ((suffix == 'H' && lsuffix == 'H') || (suffix == 'h' && lsuffix == 'h')) {
+        argv[i][strlen(argv[i]) - 2] = '\0';
+        data.size = 1;
+      } else if (suffix == 'H' || suffix == 'h' || suffix == 'S' || suffix == 's') {
+        argv[i][strlen(argv[i]) - 1] = '\0';
+        data.size = 2;
+      } else if (suffix == '\'') {
+        data.size = 1;
+      }
+
+      // Try integers
+      data.ll = strtoll(argv[i], &end_ptr, 0);
+      if (*end_ptr || (end_ptr == argv[i])) {
+        // Try float
+        data.f = strtof(argv[i], &end_ptr);
+        data.is_float = true;
+        if (*end_ptr || (end_ptr == argv[i])) {
+          data.is_float = false;
+          // Try single character
+          if (argv[i][0] == '\'' && argv[i][2] == '\'') {
+            data.ll = argv[i][1];
+          } else {
+            // Try string that starts and ends with quotes
+            if (argv[i][0] == '\"' && argv[i][strlen(argv[i]) - 1] == '\"') {
+              data.str_ptr = calloc(strlen(argv[i]), sizeof(char));
+              if (data.str_ptr == NULL) {
+                avrdude_message(MSG_INFO, "%s (write str): out of memory\n", progname);
+                return -1;
+              }
+              // Strip start and end quotes
+              strncpy(data.str_ptr, argv[i] + 1, strlen(argv[i]) - 2);
+            } else {
+            avrdude_message(MSG_INFO, "\n%s (write): can't parse data '%s'\n",
+                            progname, argv[i]);
+            free(buf);
+            if(data.str_ptr != NULL)
+              free(data.str_ptr);
+            return -1;
+            }
+          }
+        }
+      }
+
+      // Print warning if data size might be ambiguous
+      bool is_hex       = (strncmp(argv[i], "0x",  2) == 0);
+      bool is_neg_hex   = (strncmp(argv[i], "-0x", 3) == 0);
+      bool leading_zero = (strncmp(argv[i], "0x0", 3) == 0);
+      int8_t hex_digits = (strlen(argv[i]) - 2);
+      if(!data.size                                                                      // No pre-defined size
+        && (is_neg_hex                                                                   // Hex with - sign in front
+        || (is_hex && leading_zero && (hex_digits & (hex_digits - 1)))                   // Hex with 3, 5, 6 or 7 digits
+        || (!is_hex && !data.is_float && llabs(data.ll) > 0xFF && strlen(argv[i]) > 2))) // Base10 int greater than 255
+      {
+        avrdude_message(MSG_INFO, "Warning: no size suffix specified for \"%s\". "
+                                  "Writing %d byte(s)\n",
+                                  argv[i],
+                                  llabs(data.ll) > UINT32_MAX ? 8 :
+                                  llabs(data.ll) > UINT16_MAX || data.is_float ? 4 : \
+                                  llabs(data.ll) > UINT8_MAX ? 2 : 1);
+      }
+      // Flag if signed integer and adjust size
+      if (data.ll < 0 && !data.is_float) {
+        data.is_signed = true;
+        if (data.ll < INT32_MIN)
+          data.size = 8;
+        else if (data.ll < INT16_MIN)
+          data.size = 4;
+        else if (data.ll < INT8_MIN)
+          data.size = 2;
+        else
+          data.size = 1;
+      }
+    }
+    if(data.str_ptr) {
+      for(int16_t j = 0; j < strlen(data.str_ptr); j++)
+        buf[i - start_offset + data.bytes_grown++] = (uint8_t)data.str_ptr[j];
+    } else {
+      buf[i - start_offset + data.bytes_grown]     = data.a[0];
+      if (llabs(data.ll) > 0x000000FF || data.size >= 2 || data.is_float)
+        buf[i - start_offset + ++data.bytes_grown] = data.a[1];
+      if (llabs(data.ll) > 0x0000FFFF || data.size >= 4 || data.is_float) {
+        buf[i - start_offset + ++data.bytes_grown] = data.a[2];
+        buf[i - start_offset + ++data.bytes_grown] = data.a[3];
+      }
+      if (llabs(data.ll) > 0xFFFFFFFF || data.size == 8) {
+        buf[i - start_offset + ++data.bytes_grown] = data.a[4];
+        buf[i - start_offset + ++data.bytes_grown] = data.a[5];
+        buf[i - start_offset + ++data.bytes_grown] = data.a[6];
+        buf[i - start_offset + ++data.bytes_grown] = data.a[7];
+      }
+    }
+
+    // Make sure buf does not overflow
+    if (i - start_offset + data.bytes_grown > maxsize)
+      break;
+  }
+
+  // When in "fill" mode, the maximum size is already predefined
+  if (write_mode == WRITE_MODE_FILL)
+    data.bytes_grown = 0;
+
+  if ((addr + len + data.bytes_grown) > maxsize) {
+    avrdude_message(MSG_INFO, "%s (write): selected address and # bytes exceed "
+                    "range for %s memory\n",
+                    progname, memtype);
+    free(buf);
+    return -1;
+  }
+
+  if(data.str_ptr)
+    free(data.str_ptr);
+
+  avrdude_message(MSG_NOTICE, "\nInfo: Writing %d bytes starting from address 0x%02x",
+                  len + data.bytes_grown, addr);
+  if (write_mode == WRITE_MODE_FILL)
+    avrdude_message(MSG_NOTICE, ". Remaining space filled with %s", argv[argc - 2]);
+  avrdude_message(MSG_NOTICE, "\n");
+
+  pgm->err_led(pgm, OFF);
+  bool werror = false;
+  report_progress(0, 1, "Writing");
+  for (i = 0; i < (len + data.bytes_grown); i++) {
+    int32_t rc = avr_write_byte(pgm, p, mem, addr+i, buf[i]);
     if (rc) {
       avrdude_message(MSG_INFO, "%s (write): error writing 0x%02x at 0x%05lx, rc=%d\n",
               progname, buf[i], addr+i, rc);
       if (rc == -1)
         avrdude_message(MSG_INFO, "write operation not supported on memory type \"%s\"\n",
                         mem->desc);
-      werror = 1;
+      werror = true;
     }
 
+    uint8_t b;
     rc = pgm->read_byte(pgm, p, mem, addr+i, &b);
     if (b != buf[i]) {
       avrdude_message(MSG_INFO, "%s (write): error writing 0x%02x at 0x%05lx cell=0x%02x\n",
                       progname, buf[i], addr+i, b);
-      werror = 1;
+      werror = true;
     }
 
     if (werror) {
       pgm->err_led(pgm, ON);
     }
+
+    report_progress(i, (len + data.bytes_grown), NULL);
   }
+  report_progress(1, 1, NULL);
 
   free(buf);
 
-  fprintf(stdout, "\n");
-
   return 0;
 }
 
@@ -543,6 +713,10 @@ static int cmd_sig(PROGRAMMER * pgm, struct avrpart * p,
 static int cmd_quit(PROGRAMMER * pgm, struct avrpart * p,
 		    int argc, char * argv[])
 {
+  /* FUSE bit verify will fail if left in SPI mode */
+  if (spi_mode) {
+    cmd_pgm(pgm, p, 0, NULL);
+  }
   return 1;
 }
 
@@ -733,18 +907,26 @@ static int cmd_help(PROGRAMMER * pgm, struct avrpart * p,
 static int cmd_spi(PROGRAMMER * pgm, struct avrpart * p,
         int argc, char * argv[])
 {
-  pgm->setpin(pgm, PIN_AVR_RESET, 1);
-  spi_mode = 1;
-  return 0;
+  if (pgm->setpin != NULL) {
+    pgm->setpin(pgm, PIN_AVR_RESET, 1);
+    spi_mode = 1;
+    return 0;
+  }
+  avrdude_message(MSG_INFO, "`spi' command unavailable for this programmer type\n");
+  return -1;
 }
 
 static int cmd_pgm(PROGRAMMER * pgm, struct avrpart * p,
         int argc, char * argv[])
 {
-  pgm->setpin(pgm, PIN_AVR_RESET, 0);
-  spi_mode = 0;
-  pgm->initialize(pgm, p);
-  return 0;
+  if (pgm->setpin != NULL) {
+    pgm->setpin(pgm, PIN_AVR_RESET, 0);
+    spi_mode = 0;
+    pgm->initialize(pgm, p);
+    return 0;
+  }
+  avrdude_message(MSG_INFO, "`pgm' command unavailable for this programmer type\n");
+  return -1;
 }
 
 static int cmd_verbose(PROGRAMMER * pgm, struct avrpart * p,
@@ -819,12 +1001,26 @@ static int tokenize(char * s, char *** argv)
     nbuf[0]  = 0;
     n++;
     if ((n % 20) == 0) {
+      char *buf_tmp;
+      char **bufv_tmp;
       /* realloc space for another 20 args */
       bufsize += 20;
       nargs   += 20;
       bufp     = buf;
-      buf      = realloc(buf, bufsize);
-      bufv     = realloc(bufv, nargs*sizeof(char *));
+      buf_tmp  = realloc(buf, bufsize);
+      if (buf_tmp == NULL) {
+        free(buf);
+        free(bufv);
+        return -1;
+      }
+      buf = buf_tmp;
+      bufv_tmp = realloc(bufv, nargs*sizeof(char *));
+      if (bufv_tmp == NULL) {
+        free(buf);
+        free(bufv);
+        return -1;
+      }
+      bufv = bufv_tmp;
       nbuf     = &buf[l];
       /* correct bufv pointers */
       k = buf - bufp;
@@ -897,7 +1093,7 @@ static int do_cmd(PROGRAMMER * pgm, struct avrpart * p,
 
 char * terminal_get_input(const char *prompt)
 {
-#if defined(HAVE_LIBREADLINE) && !defined(WIN32NATIVE)
+#if defined(HAVE_LIBREADLINE) && !defined(WIN32)
   char *input;
   input = readline(prompt);
   if ((input != NULL) && (strlen(input) >= 1))
@@ -942,6 +1138,10 @@ int terminal_mode(PROGRAMMER * pgm, struct avrpart * p)
 
     /* tokenize command line */
     argc = tokenize(q, &argv);
+    if (argc < 0) {
+      free(cmdbuf);
+      return argc;
+    }
 
     fprintf(stdout, ">>> ");
     for (i=0; i<argc; i++)
