@@ -16,18 +16,22 @@
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: term.h 1294 2014-03-12 23:03:18Z joerg_wunsch $ */
+/* $Id$ */
 
 #ifndef term_h
 #define term_h
 
-#include "avr.h"
-#include "pgm.h"
+#include "libavrdude.h"
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+typedef enum {
+  WRITE_MODE_STANDARD = 0,
+  WRITE_MODE_FILL     = 1,
+} mode;
+
 int terminal_mode(PROGRAMMER * pgm, struct avrpart * p);
 char * terminal_get_input(const char *prompt);
 
